@@ -40,6 +40,11 @@ def unb_ri_si(data):
             if vsc['type'] == 'ac3ph4wgf': # AC 3 phase 4 wire open loop grid former
                 ac3ph4wgf(pydgrid_obj,vsc)
                 pydgrid_obj.omega_coi_required = True
+            if vsc['type'] == 'ac3ph4wgfpi': # AC 3 phase 4 wire open loop PI-VSG grid former
+                ac3ph4wgfpi(pydgrid_obj,vsc)
+                pydgrid_obj.omega_coi_required = True
+
+
 
     if pydgrid_obj.omega_coi_required:
 
@@ -962,6 +967,8 @@ def ac3ph4wvdcq(grid,vsc_data):
     
    
     ### AC-side
+    q_a,q_b,q_c = sym.symbols(f'q_vsc_a_{bus_name}_a,q_vsc_b_{bus_name}_b,q_vsc_c_{bus_name}_c')
+
     p_ac,q_ac,p_dc,p_loss = sym.symbols(f'p_vsc_{bus_ac_name},q_vsc_{bus_ac_name},p_vsc_{bus_dc_name},p_vsc_loss_{bus_ac_name}',real=True)
     p_a_d,p_b_d,p_c_d,p_n_d = sym.symbols(f'p_a_d_{bus_ac_name},p_b_d_{bus_ac_name},p_c_d_{bus_ac_name},p_n_d_{bus_ac_name}',real=True)
     C_a,C_b,C_c = sym.symbols(f'C_a_{bus_ac_name},C_b_{bus_ac_name},C_c_{bus_ac_name}',real=True)
@@ -1028,9 +1035,9 @@ def ac3ph4wvdcq(grid,vsc_data):
     eq_i_a_r =  sym.re(s_a) - p_a_d + p_loss_a_ + p_loss_n_
     eq_i_b_r =  sym.re(s_b) - p_b_d + p_loss_b_
     eq_i_c_r =  sym.re(s_c) - p_c_d + p_loss_c_
-    eq_i_a_i =  sym.im(s_a) - C_a*q_ac
-    eq_i_b_i =  sym.im(s_b) - C_b*q_ac
-    eq_i_c_i =  sym.im(s_c) - C_c*q_ac
+    eq_i_a_i =  sym.im(s_a) - q_c
+    eq_i_b_i =  sym.im(s_b) - q_b
+    eq_i_c_i =  sym.im(s_c) - q_c
     
     eq_i_n_r = -i_n_r + i_a_r + i_b_r + i_c_r
     eq_i_n_i = -i_n_i + i_a_i + i_b_i + i_c_i
@@ -1076,7 +1083,7 @@ def ac3ph4wvdcq(grid,vsc_data):
 
     
       
-    grid.dae['u'].update({f'v_dc_{bus_dc_name}':800.0,f'q_vsc_{bus_ac_name}':0.0}) 
+    grid.dae['u'].update({f'v_dc_{bus_dc_name}':800.0,f'{str(q_a)}':0.0,f'{str(q_b)}':0.0,f'{str(q_c)}':0.0}) 
     grid.dae['u'].pop(str(v_dc_a_r))
     grid.dae['params'].update({f'a_{bus_ac_name}':a_value,f'b_{bus_ac_name}':b_value,f'c_{bus_ac_name}':c_value})
     grid.dae['params'].update({f'C_a_{bus_ac_name}':1/3,f'C_b_{bus_ac_name}':1/3,f'C_c_{bus_ac_name}':1/3})
@@ -1103,6 +1110,8 @@ def ac3ph4wvdcq2(grid,vsc_data):
     
    
     ### AC-side
+    q_a,q_b,q_c = sym.symbols(f'q_vsc_a_{bus_ac_name},q_vsc_b_{bus_ac_name},q_vsc_c_{bus_ac_name}',real=True)
+
     p_ac,q_ac,p_dc,p_loss = sym.symbols(f'p_vsc_{bus_ac_name},q_vsc_{bus_ac_name},p_vsc_{bus_dc_name},p_vsc_loss_{bus_ac_name}',real=True)
     p_a_d,p_b_d,p_c_d,p_n_d = sym.symbols(f'p_a_d_{bus_ac_name},p_b_d_{bus_ac_name},p_c_d_{bus_ac_name},p_n_d_{bus_ac_name}',real=True)
     C_a,C_b,C_c = sym.symbols(f'C_a_{bus_ac_name},C_b_{bus_ac_name},C_c_{bus_ac_name}',real=True)
@@ -1160,9 +1169,9 @@ def ac3ph4wvdcq2(grid,vsc_data):
     eq_i_a_r =  sym.re(s_a) - p_a_d + p_loss_a_ + p_loss_n_
     eq_i_b_r =  sym.re(s_b) - p_b_d + p_loss_b_
     eq_i_c_r =  sym.re(s_c) - p_c_d + p_loss_c_
-    eq_i_a_i =  sym.im(s_a) - C_a*q_ac
-    eq_i_b_i =  sym.im(s_b) - C_b*q_ac
-    eq_i_c_i =  sym.im(s_c) - C_c*q_ac
+    eq_i_a_i =  sym.im(s_a) - q_a
+    eq_i_b_i =  sym.im(s_b) - q_b
+    eq_i_c_i =  sym.im(s_c) - q_c
     
     eq_i_n_r = i_n_r + i_a_r + i_b_r + i_c_r
     eq_i_n_i = i_n_i + i_a_i + i_b_i + i_c_i
@@ -1214,7 +1223,9 @@ def ac3ph4wvdcq2(grid,vsc_data):
 
     
       
-    grid.dae['u'].update({f'v_dc_{bus_dc_name}_ref':800.0,f'q_vsc_{bus_ac_name}':0.0}) 
+    grid.dae['u'].update({f'v_dc_{bus_dc_name}_ref':800.0}) 
+    grid.dae['u'].update({f'{str(q_a)}':0.0,f'{str(q_b)}':0.0,f'{str(q_c)}':0.0}) 
+
     #grid.dae['u'].pop(str(v_dc_a_r))
     grid.dae['params'].update({f'a_{bus_ac_name}':a_value,f'b_{bus_ac_name}':b_value,f'c_{bus_ac_name}':c_value})
     grid.dae['params'].update({f'C_a_{bus_ac_name}':1/3,f'C_b_{bus_ac_name}':1/3,f'C_c_{bus_ac_name}':1/3})
@@ -1364,8 +1375,11 @@ def ac3ph4wpq(grid,vsc_data):
     ## Body:
         
     ### AC-side
-    p_ac,q_ac,p_dc,p_loss = sym.symbols(f'p_vsc_{bus_ac_name},q_vsc_{bus_ac_name},p_vsc_{bus_dc_name},p_vsc_loss_{bus_ac_name}',real=True)
-    coef_a,coef_b,coef_c = sym.symbols(f'coef_a_{bus_ac_name},coef_b_{bus_ac_name},coef_c_{bus_ac_name}',real=True)
+    p_ac_a,p_ac_b,p_ac_c = sym.symbols(f'p_vsc_a_{bus_ac_name},p_vsc_b_{bus_ac_name},p_vsc_c_{bus_ac_name}',real=True)
+    q_ac_a,q_ac_b,q_ac_c = sym.symbols(f'q_vsc_a_{bus_ac_name},q_vsc_b_{bus_ac_name},q_vsc_c_{bus_ac_name}',real=True)
+
+    p_dc,p_loss = sym.symbols(f'p_vsc_{bus_dc_name},p_vsc_loss_{bus_ac_name}',real=True)
+    #coef_a,coef_b,coef_c = sym.symbols(f'coef_a_{bus_ac_name},coef_b_{bus_ac_name},coef_c_{bus_ac_name}',real=True)
     
     v_a = v_a_r + 1j*v_a_i
     v_b = v_b_r + 1j*v_b_i
@@ -1381,12 +1395,12 @@ def ac3ph4wpq(grid,vsc_data):
     s_b = (v_b - v_n) * sym.conjugate(i_b)
     s_c = (v_c - v_n) * sym.conjugate(i_c)
 
-    eq_i_a_r =  sym.re(s_a) - p_ac*coef_a
-    eq_i_b_r =  sym.re(s_b) - p_ac*coef_b
-    eq_i_c_r =  sym.re(s_c) - p_ac*coef_c
-    eq_i_a_i =  sym.im(s_a) - q_ac*coef_a
-    eq_i_b_i =  sym.im(s_b) - q_ac*coef_b
-    eq_i_c_i =  sym.im(s_c) - q_ac*coef_c
+    eq_i_a_r =  sym.re(s_a) - p_ac_a
+    eq_i_b_r =  sym.re(s_b) - p_ac_b
+    eq_i_c_r =  sym.re(s_c) - p_ac_c
+    eq_i_a_i =  sym.im(s_a) - q_ac_a
+    eq_i_b_i =  sym.im(s_b) - q_ac_b
+    eq_i_c_i =  sym.im(s_c) - q_ac_c
     eq_i_n_r =  sym.re(i_a+i_b+i_c+i_n)
     eq_i_n_i =  sym.im(i_a+i_b+i_c+i_n)
 
@@ -1410,6 +1424,7 @@ def ac3ph4wpq(grid,vsc_data):
     p_loss_n = a + b*i_rms_n + c*i_rms_n*i_rms_n
 
     p_vsc_loss = p_loss_a + p_loss_b + p_loss_c + p_loss_n
+    p_ac = p_ac_a + p_ac_b + p_ac_c
 
     eq_p_loss = p_loss - p_vsc_loss
     eq_i_dc_a_r = i_dc_a_r + p_dc/(v_dc_a_r-v_dc_n_r+1e-8)
@@ -1430,13 +1445,14 @@ def ac3ph4wpq(grid,vsc_data):
 
     grid.dae['g'] +=  [eq_i_dc_a_r,eq_i_dc_n_r,eq_p_dc,eq_p_loss]
     grid.dae['y'] +=  [   i_dc_a_r,   i_dc_n_r,   p_dc,   p_loss]  
-    grid.dae['u'].update({f'p_vsc_{bus_ac_name}':0.0,f'q_vsc_{bus_ac_name}':0.0}) 
-    grid.dae['xy_0_dict'].update({f'v_{bus_dc_name}_a_r':800.0,f'v_{bus_dc_name}_n_r':10.0})
+    #grid.dae['u'].update({f'p_vsc_{bus_ac_name}':0.0,f'q_vsc_{bus_ac_name}':0.0}) 
+    grid.dae['xy_0_dict'].update({f'v_{bus_dc_name}_a_r':800.0,f'v_{bus_dc_name}_n_r':0.01})
     
     #grid.dae['u'].pop(f'i_{bus_dc_name}_a_r')
     #grid.dae['u'].pop(f'i_{bus_dc_name}_n_r')
     grid.dae['params'].update({f'a_{bus_ac_name}':a_value,f'b_{bus_ac_name}':b_value,f'c_{bus_ac_name}':c_value})
-    grid.dae['params'].update({f'coef_a_{bus_ac_name}':1/3,f'coef_b_{bus_ac_name}':1/3,f'coef_c_{bus_ac_name}':1/3})
+    grid.dae['u'].update({f'{str(p_ac_a)}':0.0,f'{str(p_ac_b)}':0.0,f'{str(p_ac_c)}':0.0})
+    grid.dae['u'].update({f'{str(q_ac_a)}':0.0,f'{str(q_ac_b)}':0.0,f'{str(q_ac_c)}':0.0})
 
     ## Add current injections to grid equations:
     bus_idx = grid.nodes.index(f'{bus_dc_name}.{1}')
@@ -1605,6 +1621,10 @@ def ac3ph4wgf(grid,vsc_data):
     
     # dynamical states
     phi = sym.Symbol(f'phi_{name}', real=True)
+    phi_a = sym.Symbol(f'phi_a_{name}', real=True)
+    phi_b = sym.Symbol(f'phi_b_{name}', real=True)
+    phi_c = sym.Symbol(f'phi_c_{name}', real=True)
+
     omega = sym.Symbol(f'omega_{name}', real=True)
     
     # algebraic states
@@ -1638,12 +1658,12 @@ def ac3ph4wgf(grid,vsc_data):
     v_ng = v_ng_r + 1j*v_ng_i
     v_mn = v_mn_r + 1j*v_mn_i
     
-    e_am_r = e_am_m*sym.cos(phi) 
-    e_am_i = e_am_m*sym.sin(phi) 
-    e_bm_r = e_bm_m*sym.cos(phi-2/3*np.pi) 
-    e_bm_i = e_bm_m*sym.sin(phi-2/3*np.pi) 
-    e_cm_r = e_cm_m*sym.cos(phi-4/3*np.pi) 
-    e_cm_i = e_cm_m*sym.sin(phi-4/3*np.pi) 
+    e_am_r = e_am_m*sym.cos(phi_a + phi) 
+    e_am_i = e_am_m*sym.sin(phi_a + phi) 
+    e_bm_r = e_bm_m*sym.cos(phi_b + phi-2/3*np.pi) 
+    e_bm_i = e_bm_m*sym.sin(phi_b + phi-2/3*np.pi) 
+    e_cm_r = e_cm_m*sym.cos(phi_c + phi-4/3*np.pi) 
+    e_cm_i = e_cm_m*sym.sin(phi_c + phi-4/3*np.pi) 
     
     e_am_cplx = e_am_r + 1j*e_am_i
     e_bm_cplx = e_bm_r + 1j*e_bm_i
@@ -1702,7 +1722,11 @@ def ac3ph4wgf(grid,vsc_data):
     #e_an_i,e_bn_i,e_cn_i = V_abc.imag
 
     u_dict.update({f'e_{name}_am_m':V_1,f'e_{name}_bm_m':V_1,f'e_{name}_cm_m':V_1,f'e_{name}_om_m':0.0})
-    u_dict.update({f'phi_{name}':0.0})
+    #u_dict.update({f'phi_{name}':0.0})
+    u_dict.update({f'phi_a_{name}':0.0})
+    u_dict.update({f'phi_b_{name}':0.0})
+    u_dict.update({f'phi_c_{name}':0.0})
+
     u_dict.update({f'p_{name}_ref':0.0})
     u_dict.update({f'omega_{name}_ref':1.0})
 
@@ -1749,9 +1773,237 @@ def ac3ph4wgf(grid,vsc_data):
     grid.omega_coi_h_i += omega_coi_i
     grid.hs_total += HS_coi
 
+def ac3ph4wgfpi(grid,vsc_data):
+    '''
+    VSC with 3 phase and 4 wire working in open loop as a grid former.
+
+    
+    '''
+
+    params_dict  = grid.dae['params']
+    f_list = grid.dae['f']
+    x_list = grid.dae['x']
+    g_list = grid.dae['g'] 
+    y_list = grid.dae['y'] 
+    u_dict = grid.dae['u']
+    h_dict = grid.dae['h_dict']
 
 
+    alpha = np.exp(2.0/3*np.pi*1j)
+    A_0a =  np.array([[1, 1, 1],
+                    [1, alpha**2, alpha],
+                    [1, alpha, alpha**2]])
+
+    A_a0 = 1/3* np.array([[1, 1, 1],
+                        [1, alpha, alpha**2],
+                        [1, alpha**2, alpha]])
+
+    omega_coi_i = 0
+    HS_coi = 0
+
+    omega_coi = sym.Symbol('omega_coi',real=True)
+    xi_freq = sym.Symbol('xi_freq',real=True)
+    K_agc = sym.Symbol('K_agc',real=True)
+
+    #vscs = [
+    #    {'bus':'B1','S_n':100e3,'R':0.01,'X':0.1,'R_n':0.01,'X_n':0.1,'R_ng':0.01,'X_ng':3.0,'K_f':0.1,'T_f':1.0,'K_sec':0.5,'K_delta':0.001},
+    #    ]
+
+    #for vsc in vsc_data:
+        
+    name = vsc_data['bus']
+
+    # inputs
+    e_am_m,e_bm_m,e_cm_m,e_om_m = sym.symbols(f'e_{name}_am_m,e_{name}_bm_m,e_{name}_cm_m,e_{name}_om_m', real=True)
+    omega_ref,p_c = sym.symbols(f'omega_{name}_ref,p_c_{name}', real=True)
+    phi_a = sym.Symbol(f'phi_a_{name}', real=True)
+    phi_b = sym.Symbol(f'phi_b_{name}', real=True)
+    phi_c = sym.Symbol(f'phi_c_{name}', real=True)
+
+    # parameters
+    S_n,H,K_f,T_f,K_sec,K_delta  = sym.symbols(f'S_n_{name},H_{name},K_f_{name},T_f_{name},K_sec_{name},K_delta_{name}', real=True)
+    R_s,R_sn,R_ng = sym.symbols(f'R_{name}_s,R_{name}_sn,R_{name}_ng', real=True)
+    X_s,X_sn,X_ng = sym.symbols(f'X_{name}_s,X_{name}_sn,X_{name}_ng', real=True)
+    T_e = sym.Symbol(f'T_e_{name}', real=True)
+    K_p = sym.Symbol(f"K_p_{name}", real=True)    
+    T_p = sym.Symbol(f"T_p_{name}", real=True)   
+    T_c = sym.Symbol(f"T_c_{name}", real=True)   
+
+    Droop = sym.Symbol(f"Droop_{name}", real=True)   
+    
+    # dynamical states
+    phi = sym.Symbol(f'phi_{name}', real=True)
+    xi_p = sym.Symbol(f'xi_p_{name}', real=True)
+    p_ef = sym.Symbol(f'p_ef_{name}', real=True)
+
+
+
+    omega = sym.Symbol(f'omega_{name}', real=True)
+    
+    # algebraic states
+    #e_an_i,e_bn_i,e_cn_i,e_ng_i = sym.symbols(f'e_{name}_an_i,e_{name}_bn_i,e_{name}_cn_i,e_{name}_ng_i', real=True)
+    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_ng_r = sym.symbols(f'v_{name}_a_r,v_{name}_b_r,v_{name}_c_r,v_{name}_n_r,v_{name}_n_r', real=True)
+    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_ng_i = sym.symbols(f'v_{name}_a_i,v_{name}_b_i,v_{name}_c_i,v_{name}_n_i,v_{name}_n_i', real=True)
+    i_sa_r,i_sb_r,i_sc_r,i_sn_r,i_ng_r = sym.symbols(f'i_vsc_{name}_a_r,i_vsc_{name}_b_r,i_vsc_{name}_c_r,i_vsc_{name}_n_r,i_vsc_{name}_ng_r', real=True)
+    i_sa_i,i_sb_i,i_sc_i,i_sn_i,i_ng_i = sym.symbols(f'i_vsc_{name}_a_i,i_vsc_{name}_b_i,i_vsc_{name}_c_i,i_vsc_{name}_n_i,i_vsc_{name}_ng_i', real=True)
+    v_mn_r,v_mn_i = sym.symbols(f'v_{name}_mn_r,v_{name}_mn_i', real=True)
+
+    omega = sym.Symbol(f'omega_{name}', real=True)
+    p_ef  = sym.Symbol(f'p_ef_{name}', real=True)
+    p_m = sym.Symbol(f'p_m_{name}', real=True) 
+    p_c = sym.Symbol(f'p_c_{name}', real=True) 
+    p_cf= sym.Symbol(f'p_cf_{name}', real=True)   
+
+    e_om_r,e_om_i = sym.symbols(f'e_{name}_om_r,e_{name}_om_i', real=True)
+    
+    Z_sa = R_s + 1j*X_s
+    Z_sb = R_s + 1j*X_s
+    Z_sc = R_s + 1j*X_s
+    Z_sn = R_sn + 1j*X_sn
+    Z_ng = R_ng + 1j*X_ng
+
+    i_sa = i_sa_r + 1j*i_sa_i
+    i_sb = i_sb_r + 1j*i_sb_i
+    i_sc = i_sc_r + 1j*i_sc_i
+    i_sn = i_sn_r + 1j*i_sn_i
+    i_ng = i_ng_r + 1j*i_ng_i
+
+    v_sa = v_sa_r + 1j*v_sa_i
+    v_sb = v_sb_r + 1j*v_sb_i
+    v_sc = v_sc_r + 1j*v_sc_i
+    v_sn = v_sn_r + 1j*v_sn_i
+    v_ng = v_ng_r + 1j*v_ng_i
+    v_mn = v_mn_r + 1j*v_mn_i
+    
+    e_am_r = e_am_m*sym.cos(phi_a + phi) 
+    e_am_i = e_am_m*sym.sin(phi_a + phi) 
+    e_bm_r = e_bm_m*sym.cos(phi_b + phi-2/3*np.pi) 
+    e_bm_i = e_bm_m*sym.sin(phi_b + phi-2/3*np.pi) 
+    e_cm_r = e_cm_m*sym.cos(phi_c + phi-4/3*np.pi) 
+    e_cm_i = e_cm_m*sym.sin(phi_c + phi-4/3*np.pi) 
+    
+    e_am_cplx = e_am_r + 1j*e_am_i
+    e_bm_cplx = e_bm_r + 1j*e_bm_i
+    e_cm_cplx = e_cm_r + 1j*e_cm_i
+    e_om_cplx = e_om_r + 1j*e_om_i
+
+    v_san = v_sa - v_sn
+    v_sbn = v_sb - v_sn
+    v_scn = v_sc - v_sn
+
+    eq_i_sa_cplx = e_am_cplx - i_sa*Z_sa - v_san - v_mn
+    eq_i_sb_cplx = e_bm_cplx - i_sb*Z_sb - v_sbn - v_mn
+    eq_i_sc_cplx = e_cm_cplx - i_sc*Z_sc - v_scn - v_mn
+    eq_v_nm_cplx = 0*e_om_cplx - i_sn*Z_sn - v_mn
+    eq_i_sn_cplx = i_sa + i_sb + i_sc + i_sn
+    #eq_i_sn_cplx = e_ng_cplx - i_sn*Z_sn - v_ng
+    #eq_i_ng_cplx = i_ng + i_sa + i_sb + i_sc + i_sn
+    #eq_e_ng_cplx  = -e_ng_cplx  + i_ng*Z_ng
+
+    g_list += [sym.re(eq_i_sa_cplx)] 
+    g_list += [sym.re(eq_i_sb_cplx)] 
+    g_list += [sym.re(eq_i_sc_cplx)] 
+    g_list += [sym.re(eq_v_nm_cplx)] 
+    g_list += [sym.re(eq_i_sn_cplx)] 
+    
+    #g_list += [sym.re(eq_i_ng_cplx)] 
+    #g_list += [sym.re(eq_e_ng_cplx)] 
+    g_list += [sym.im(eq_i_sa_cplx)] 
+    g_list += [sym.im(eq_i_sb_cplx)] 
+    g_list += [sym.im(eq_i_sc_cplx)] 
+    g_list += [sym.im(eq_v_nm_cplx)] 
+    g_list += [sym.im(eq_i_sn_cplx)] 
+    #g_list += [sym.im(eq_i_ng_cplx)] 
+    #g_list += [sym.im(eq_e_ng_cplx)]
+
+    y_list += [i_sa_r,i_sb_r,i_sc_r,v_mn_r,i_sn_r]
+    y_list += [i_sa_i,i_sb_i,i_sc_i,v_mn_i,i_sn_i]
+
+    y_ini_str = [str(item) for item in y_list]
+
+    for ph in ['a','b','c','n']:
+        i_s_r = sym.Symbol(f'i_vsc_{name}_{ph}_r', real=True)
+        i_s_i = sym.Symbol(f'i_vsc_{name}_{ph}_i', real=True)  
+        g_list[y_ini_str.index(f'v_{name}_{ph}_r')] += i_s_r
+        g_list[y_ini_str.index(f'v_{name}_{ph}_i')] += i_s_i
+        i_s = i_s_r + 1j*i_s_i
+        i_s_m = np.abs(i_s)
+        h_dict.update({f'i_vsc_{name}_{ph}_m':i_s_m})
+
+        
+    V_1 = 400/np.sqrt(3)
+    #    V_1 = 400/np.sqrt(3)*np.exp(1j*np.deg2rad(0))
+    # A_1toabc = np.array([1, alpha**2, alpha])
+    #V_abc = V_1 * A_1toabc 
+    #e_an_r,e_bn_r,e_cn_r = V_abc.real
+    #e_an_i,e_bn_i,e_cn_i = V_abc.imag
+
+    u_dict.update({f'e_{name}_am_m':V_1,f'e_{name}_bm_m':V_1,f'e_{name}_cm_m':V_1,f'e_{name}_om_m':0.0})
+    #u_dict.update({f'phi_{name}':0.0})
+    u_dict.update({f'phi_a_{name}':0.0})
+    u_dict.update({f'phi_b_{name}':0.0})
+    u_dict.update({f'phi_c_{name}':0.0})
+
+    u_dict.update({f'p_c_{name}':0.0})
+    u_dict.update({f'omega_{name}_ref':1.0})
+
+    #for ph in ['a','b','c','n']:
+    #    u_dict.pop(f'i_{name}_{ph}_r')
+    #    u_dict.pop(f'i_{name}_{ph}_i')
+
+    params_dict.update({f'X_{name}_s':vsc_data['X'],f'R_{name}_s':vsc_data['R']})
+    params_dict.update({f'X_{name}_sn':vsc_data['X_n'],f'R_{name}_sn':vsc_data['R_n']})
+    params_dict.update({f'X_{name}_ng':vsc_data['X_ng'],f'R_{name}_ng':vsc_data['R_ng']})
+    
+    params_dict.update({f'S_n_{name}':vsc_data['S_n']})
+    params_dict.update({f'T_e_{name}':vsc_data['T_e']})
+    params_dict.update({f'T_c_{name}':vsc_data['T_c']})
+    params_dict.update({f'Droop_{name}':vsc_data['Droop']})
+    params_dict.update({f'K_p_{name}':vsc_data['K_p']})
+    params_dict.update({f'T_p_{name}':vsc_data['T_p']})
+    params_dict.update({f'K_sec_{name}':vsc_data['K_sec']})
+    params_dict.update({f'K_delta_{name}':vsc_data['K_delta']})
+    
+    
+    v_sabc = sym.Matrix([[v_sa],[v_sb],[v_sc]])
+    i_sabc = sym.Matrix([[i_sa],[i_sb],[i_sc]])
+    
+    v_szpn = A_a0*v_sabc
+    i_szpn = A_a0*i_sabc
+    
+    s_pos = 3*v_szpn[1]*sym.conjugate(i_szpn[1])
+    s_neg = 3*v_szpn[2]*sym.conjugate(i_szpn[2])
+    s_zer = 3*v_szpn[0]*sym.conjugate(i_szpn[0])
+    
+    p_pos = sym.re(s_pos)
+
+    p_agc = K_agc*xi_freq
+    p_r = K_sec*p_agc
+    epsilon_p = p_m - p_ef
+
+    dphi   = 2*np.pi*50*(omega - omega_coi) - K_delta*phi
+    dxi_p = epsilon_p
+    dp_ef = 1/T_e*(p_pos/S_n - p_ef)
+    dp_cf = 1/T_c*(p_c - p_cf)
    
+    # algebraic equations   
+    g_omega = -omega + K_p*(epsilon_p + xi_p/T_p) + 1
+    g_p_m  = -p_m + p_cf + p_r - 1/Droop*(omega - omega_ref)
+
+    g_list += [g_omega, g_p_m] 
+    y_list += [  omega,   p_m] 
+
+    f_list += [dphi,dxi_p,dp_ef,dp_cf]
+    x_list += [ phi, xi_p, p_ef, p_cf]
+    
+    h_dict.update({f'p_{name}_pos':sym.re(s_pos),f'p_{name}_neg':sym.re(s_neg),f'p_{name}_zer':sym.re(s_zer)})
+    h_dict.update({str(e_am_m):e_am_m,str(e_bm_m):e_bm_m,str(e_cm_m):e_cm_m})
+    h_dict.update({str(p_c):p_c,str(omega_ref):omega_ref})
+    HS_coi  = S_n
+    omega_coi_i = S_n*omega
+
+    grid.omega_coi_h_i += omega_coi_i
+    grid.hs_total += HS_coi
 
 
 
